param (
    [switch]$CpuTemperature = $false,
    [switch]$RunningServices = $false,
    [switch]$SmartDiskCheck = $false,
    [switch]$DiskClean = $false,
    [switch]$PupUninstaller = $false,
    [switch]$CheckAV = $false,
    [switch]$CheckPageFile = $false, 
    [switch]$VisualPerformance = $false,
    [switch]$FixPrivacy = $false,
    [switch]$InstallPackages = $false,
    [switch]$DisableSystemServices = $false,
    [switch]$DisableScheduledTasks = $false,
    [switch]$UninstallDefaultApps = $false,
    [switch]$PowerOptions = $false,
    [switch]$SystemRestore = $false,
    [switch]$CreateScheduledTasks = $false,
    [switch]$ImportScheduledTasks = $false,
    [switch]$PersistantFiles = $false,
    [switch]$CopyFiles = $false,
    [switch]$UserAccountControll = $false,
    [switch]$CheckAvStatus = $false,
    [switch]$CheckAvDefs = $false,
    [switch]$CheckDns = $false,
	[switch]$GeneralTweaksApply = $false,
	[switch]$UpdatePrograms = $false,
    [switch]$test = $false
)

#import files
$ScriptDirectory = Split-Path -Path $MyInvocation.MyCommand.Definition -Parent
try
{
	. ("$ScriptDirectory\Functions\ChangeServiceState.ps1")
	. ("$ScriptDirectory\Functions\CheckServices.ps1")
	. ("$ScriptDirectory\Functions\CopyFiles.ps1")
	. ("$ScriptDirectory\Functions\DiskCleanUp.ps1")
	. ("$ScriptDirectory\Functions\GetFailingDrive.ps1")
	. ("$ScriptDirectory\Functions\GetInstalledPrograms.ps1")
	. ("$ScriptDirectory\Functions\GetTemperature.ps1")
	. ("$ScriptDirectory\Functions\IpDns.ps1")
	. ("$ScriptDirectory\Functions\MessageBox.ps1")
	. ("$ScriptDirectory\Functions\PackageManager.ps1")
	. ("$ScriptDirectory\Functions\PageFile.ps1")
	. ("$ScriptDirectory\Functions\PowerManagment.ps1")
	. ("$ScriptDirectory\Functions\Registry.ps1")
	. ("$ScriptDirectory\Functions\SendMail.ps1")
	. ("$ScriptDirectory\Settings.ps1")
	. ("$ScriptDirectory\Functions\TaskSchedular.ps1")
	. ("$ScriptDirectory\Functions\Uac.ps1")
	. ("$ScriptDirectory\Functions\Uninstaller.ps1")
	
}
catch
{
	Write-Host "Error while loading supporting PowerShell Scripts"
}

if ($test)
{
	
}

#Persistantly copy files
if ($PersistantFiles) {
        foreach($PersistantCopy in $PersistantFilesCopy){
            CopyFiles -CopyFile $PersistantCopy
        }
}


#copy files one time only
if ($CopyFiles) {
        foreach($CopyFile in $CopyFilesOnce){
            CopyFiles -CopyFile $CopyFile
        }
}

#uac control
if ($UserAccountControll)
{
	Write-Output $Uac
	if ($Uac -eq "False")
	{
        DisableUac
    }
	else
	{
        EnableUac
    }
}

#DNS Check
if($CheckDns){
    SetDns -Adapters $ActiveNetworkAdapters -DnsSettings $DnsAddress
}

# Checks for a service to be running.
if ($RunningServices) {

    if($ServiceMonitorEnabled -eq "True"){

        $StoppedServices = @()

        Foreach ($Service in $Services)
        {
            $Result = (CheckService -ServiceName $Service)

            If ($Result){

                $StoppedServices += ,$Service 
                   
            }
        }

        If ($StoppedServices -ne $Null) {
            if ($EmailEnabled){
                SendMail -Subject "$Computername : a service is not running." -Body "Service $StoppedServices not running on $Computername"
            }
            else{
                SimpleMessageBox -MessageBoxInfo "$StoppedServices has stopped running. Contact $CompanyName for help $CompanyTel $CompanyEmail"
            }
        
        }

    }

}


# Checks for dying HDD.
if ($SmartDiskCheck) {

        $Result = (GetFailingDrive)

        If ($Result){
            if ($EmailEnabled){
                SendMail -Subject "$Computername : Failing HDD." -Body "Failing HDD on $Computername $Result"
            }
            else{
                SimpleMessageBox -MessageBoxInfo "Your Hard Drive is failing.  Backup your information immediately. Contact $CompanyName for help $CompanyTel $CompanyEmail"
            }
                   
        }

}

# Checks CPU temperature.
if ($CpuTemperature) {  
        $IsCpuTempCritical = CriticalCpuTemperature -MaxCpuTemperature $MaxCpuTemperature
        if ($IsCpuTempCritical){
            if ($EmailEnabled){
                SendMail -Subject "$Computername : CPU is running hot." -Body "$Computername CPU is running hot"
            }
            else{
                SimpleMessageBox -MessageBoxInfo "Your Computer is running hot, damage may occur. Contact $CompanyName for help $CompanyTel $CompanyEmail"
            }
            
    }
}


#Disk clean up
if ($DiskClean) {
    RemoveTemp -Folders $TempFolders
    WindowsDiskCleanUp
    RecycleBinEmpty -RetentionTime $RecycleBinRetentionTime -DeleteItems
}

#unwanted programs uninstall
if ($PupUninstaller) {
    $programs = (GetPrograms)
    UninstallPup -ProgramList $programs -PupList $PupList
}


#Check Antivirus
if ($CheckAV) {
    GetAntiVirusProduct
}

#check anti virus status
if ($CheckAvStatus){

    if($AvMonitorEnabled -eq "True"){

        $IsAntiVirusInstalled = CheckAntiVirusStatus -InstalledAV $InstalledAntiVirus

        if($IsAntiVirusInstalled){
            if ($EmailEnabled){
                    SendMail -Subject "$Computername : No Anti Virus." -Body "No Anti Virus $Computername $Result"
                }
                else{
                    SimpleMessageBox -MessageBoxInfo "Your AntiVirus has changed or stopped working. Contact $CompanyName for help $CompanyTel $CompanyEmail"
                }
        }

    }

}

#check anti virus defs
if ($CheckAvDefs){

    if($AvMonitorEnabled -eq "True"){
     
        $AntiVirusDefinitions = CheckAntiVirusDefinitions

        if($AntiVirusDefinitions){
            if ($EmailEnabled){
                    SendMail -Subject "$Computername : Out of date virus definitions." -Body "Out of date virus definitions $Computername $Result"
                }
                else{
                    SimpleMessageBox -MessageBoxInfo "Your AntiVirus definitions are out of date. Try updating Your AntiVirus if that fails your AntiVirus may have stopped working. Contact $CompanyName for help $CompanyTel $CompanyEmail"
                }
        }

    }

}

#Check Page File
if ($CheckPageFile) {
    PageSizeOptimal -pageSize $pageSize   
}

#adjust visual for best performance rather than looks
if ($VisualPerformance) {
    ModifyRegistryKey -RegKeys $VisualPerformanceSettings -DefaultValue $VisualPerformanceDefaults
}

#fix privacy
if ($FixPrivacy) {
    foreach ($key in (Get-ChildItem "HKCU:\SOFTWARE\Microsoft\Windows\CurrentVersion\BackgroundAccessApplications")) {
		Set-ItemProperty ("HKCU:\SOFTWARE\Microsoft\Windows\CurrentVersion\BackgroundAccessApplications\" + $key.PSChildName) "Disabled" $BackgroundAppsPrivacy
		Set-ItemProperty ("HKCU:\SOFTWARE\Microsoft\Windows\CurrentVersion\BackgroundAccessApplications\" + $key.PSChildName) "DisabledByUser" $BackgroundAppsPrivacy
    }
    ModifyRegistryKey -RegKeys $PrivacySettings -DefaultValue $PrivacyDefaults
}

if($GeneralTweaksApply){
    ModifyRegistryKey -RegKeys $GeneralTweaks -DefaultValue $GeneralTweaksDefaults
}


#package manager
if ($InstallPackages) {
	PackageManagerSelect -PackageManager $PackageManager -PackagesInstall $PackagesInstall -PackagesUpdateTime $PackagesUpdateTime -updatePath $ScriptDirectory
}


#Disable Services
if ($DisableSystemServices) {
    DisableService -DisabledServices $DisabledServices
}
 

#disable scheduled tasks
if ($DisableScheduledTasks) {
    DisableSchTask -DisableSchTask -Tasks $DisableTasks
}


#uninstall default apps
if ($UninstallDefaultApps) {
    #uninstall windows apps
    UninstallWindowsApps -WindowsApps $RemoveWindowsCrapWare

    # Uninstall OneDrive
    if ($UninstallOneDrive) {
        Start-Process C:\Windows\SysWOW64\OneDriveSetup.exe /uninstall -Wait -ErrorVariable +ProcessError
    }
}


#power options
if ($PowerOptions) {
    Set-PowerPlan -PowerPlan $PowerPlan = "high performance"
    Set-DisplaySleep -MonitorOff $MonitorOff -ComputerSleep $ComputerSleep -screenSaver $screenSaver
}


#system restore
if ($SystemRestore) {
    if($SystemRestoreState){
        Enable-ComputerRestore -Drive "C:\"
        #Set-ItemProperty -Path HKLM\SOFTWARE\Microsoft\Windows NT\CurrentVersion\SystemRestore\cfg -Name DiskPercent -Value $SystemRestoreAmmount
        Checkpoint-Computer -Description MyFirstPoint    
    }
    else{
        Disable-ComputerRestore -Drive "C:\"
    }
    
}

#scheduled tasks
if($ImportScheduledTasks){
    foreach($ImportTask in $SchedTaskImportParams){
        if($ImportTask.Enabled -eq "True"){
            TaskSchedBKRestore -BackupRestore "Restore" -Path $ImportTask.Path
        }
    }
	
}

if ($UpdatePrograms)
{
	$windowsPrograms = GetPrograms
	updateCompare -windowsPrograms $windowsPrograms -libPath "C:\ProgramData\chocolatey\lib\"
}

#if($CreateScheduledTasks){
#    foreach($task in $CreateSchTask){
#        CreateSchTask -TaskParams $task
#    }    
#}